
//		OSD.cpp  --  Plugin framework for testing the OSD command


#define UNICODE					1
#define _UNICODE				1

#include <stdio.h>
#include <tchar.h>

#include <windows.h>

#include "PlugIn.h"
#include "TakeCmd.h"


#define VER_MAJOR				1
#define VER_MINOR				0
#define VER_BUILD				11
#define VER_PATCH				6

#define PLUGIN_NAME				L"OSD"

#ifdef _WIN64
#define PLATFORM				L"x64"
#else	//	_WIN64
#define PLATFORM				L"x86"
#endif	//	_WIN64


#define MAX_ARG_LEN				8192
#define MAX_ARGS				1024
#define SHORT_BUF_LEN			1024
#define MAX_FILENAME_LEN		32768


#define OSD_FULL_HELP
#define OSD_OSD_CMD
#define OSD_PCTBAR_CMD
#define OSD_OSDIMG_CMD
#define OSD_READ_FILE
#define OSD_OSD_VARS
#define OSD_ANGLES
#define OSD_ESCAPES
#define OSD_TAB_OPTION

#define ERRHANDLE				GetStdHandle( STD_ERROR_HANDLE )

#define MACRO_EXP( arg )		#arg
#define MACRO_STR( arg )		MACRO_EXP( arg )



wchar_t PluginFeatures[] = L"osd,osdimg,pctbar,_monitors,_osd,_osdver,@osd";


const wchar_t HelpWindowTitle[] = L"OSD Plugin Help";



HINSTANCE MyDLLHandle	= NULL;



int DisplayErrorHeader()
{
	wchar_t shortbuf[MAX_ARG_LEN] = L"%_batch";

	Qprintf( ERRHANDLE, L"%s plugin: ", PLUGIN_NAME );

	ExpandVariables( shortbuf, 1 );
	if ( shortbuf[0] == '0' )
		return 0;

	wcscpy_s( shortbuf, MAX_ARG_LEN, L"%_batchname [%_batchline]  " );
	ExpandVariables( shortbuf, 1 );
	Qprintf( ERRHANDLE, shortbuf );

	return 0;
}



#include "ParseArgs.cpp"



int ParseInt( LPTSTR inString, unsigned int *outValue, wchar_t **EndChar )
{
	if ( !inString || !outValue )
		return -1;

	unsigned int value = 0;
	wchar_t *sep1 = NULL;

	_set_errno( 0 );

	if ( ( inString[0] == '0' ) && ( towupper( inString[1] ) == 'X' ) && ( iswxdigit( inString[2] ) ) ) {
		//  parse hexadecimal

		value = wcstoul( inString + 2, &sep1, 16 );

		if ( errno )
			return -1;
		else if ( *sep1 && !EndChar )
			return -1;

		*outValue = value;

		if ( EndChar )
			*EndChar  = sep1;

		return 0;
	}
	else if ( iswdigit( inString[0] ) ) {
		//  parse unsigned

		value = wcstoul( inString, &sep1, 10 );

		if ( errno )
			return -1;
		else if ( *sep1 && !EndChar )
			return -1;

		*outValue = value;

		if ( EndChar )
			*EndChar  = sep1;

		return 0;
	}

	return -1;
}



int ParseSignedInt( LPTSTR inString, int *outValue, wchar_t **EndChar )
{
	if ( !inString || !outValue )
		return -1;

	int value = 0;
	wchar_t *sep1 = NULL;

	_set_errno( 0 );

	if ( ( inString[0] == '0' ) && ( towupper( inString[1] ) == 'X' ) && ( iswxdigit( inString[2] ) ) ) {
		//  parse hexadecimal

		value = wcstoul( inString + 2, &sep1, 16 );

		if ( errno )
			return -1;
		else if ( *sep1 && !EndChar )
			return -1;

		*outValue = value;

		if ( EndChar )
			*EndChar  = sep1;

		return 0;
	}
	else if ( iswdigit( inString[0] ) || ( ( inString[0] == '-' ) && iswdigit( inString[1] ) ) ) {
		//  parse signed integer

		value = wcstol( inString, &sep1, 10 );

		if ( errno )
			return -1;
		else if ( *sep1 && !EndChar )
			return -1;

		*outValue = value;

		if ( EndChar )
			*EndChar  = sep1;

		return 0;
	}

	return -1;
}



BOOL IsColonOrEquals( wchar_t ch )
{
	return ( ( ch == ':' ) || ( ch == '=' ) );
}



int StripQuotes( LPTSTR String )
{
	if ( !String )
		return -1;

	int n = 0;

	size_t j = 0;
	for ( size_t i = 0; ; i++ ) {
		if ( String[i] == 0x22 ) {
			n++;
			continue;
		}

		String[j++] = String[i];
		if ( String[i] == '\0' )
			break;
	}

	return n;
}



LPTSTR _FilenamePart( LPTSTR Filename, LPTSTR outFilename )
{
	if ( !Filename )
		return NULL;
	else if ( Filename[0] == '\0' )
		return NULL;


	BOOL FoundSlash			= FALSE;
	size_t EndOfPathPart	= -1;


	for ( size_t i = 0; Filename[i]; i++ ) {
		if ( ( Filename[i] == '/' ) || ( Filename[i] == '\\' ) ) {
			EndOfPathPart = i;
			FoundSlash = TRUE;
		}
		else if ( ( Filename[i] == ':' ) && !FoundSlash )
			EndOfPathPart = i;
	}

	if ( EndOfPathPart == -1 ) {
		if ( outFilename ) {
			wcscpy_s( outFilename, MAX_FILENAME_LEN, Filename );
			return outFilename;
		}
		else
			return Filename;
	}

	if ( outFilename ) {
		wcscpy_s( outFilename, MAX_FILENAME_LEN, Filename + EndOfPathPart + 1 );
		return outFilename;
	}
	else
		return Filename + EndOfPathPart + 1;
}



int ShowCmdHelp( LPCTSTR HelpText, BOOL ShowVersion, LPCSTR SourceFile, LPCSTR SourceFileVer )
{
	wchar_t File[MAX_FILENAME_LEN]		= L"";
	wchar_t Ver[SHORT_BUF_LEN]			= L"";


	QPuts( (LPTSTR) HelpText );

	if ( ShowVersion ) {
		Printf( L"\r\n\t\t\t\t%s  %u.%u.%u.%u (%s)\r\n", PLUGIN_NAME, VER_MAJOR, VER_MINOR, VER_BUILD, VER_PATCH, PLATFORM );

		if ( SourceFile && SourceFileVer ) {
			MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, SourceFile, -1, File, MAX_FILENAME_LEN );
			MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, SourceFileVer, -1, Ver, SHORT_BUF_LEN );
			Printf( L"\t\t\t\t%s  %s\r\n", _FilenamePart( File, NULL), Ver );
		}
	}

	return 0;
}



//		Most of the code is pulled directly from LEStuff:

#include "OsdCmd.cpp"



BOOL PipeShell()
{
	wchar_t temp[SHORT_BUF_LEN]			= L"%[_transient]%[_pipe]";


	ExpandVariables( temp, 1 );

	if ( wcscmp( temp, L"00" ) )
		return TRUE;
	else
		return FALSE;
}



int DisableOneFeature( LPTSTR FeaturesList, LPTSTR word )
{
	if ( !FeaturesList || !word )
		return -1;

	if ( ( FeaturesList[0] == '\0' ) || ( word[0] == '\0' ) )
		return -1;


	size_t i = 0, j = 0, base = 0;
	wchar_t ch = '\0', ch2 = '\0';
	BOOL done = FALSE, match = FALSE;


	do {
		// Compare word in 'word' against word at 'base' :

		i = 0;
		match = TRUE;

		do {
			ch = towupper(FeaturesList[base + i]);
			ch2 = ch;
			if ( ( ch2 == ',' ) || ( ch2 == ';' ) )
				ch2 = '\0';

			match &= ( ch2 == towupper(word[i]) );

			i++;
		} while ( ch2 != '\0' );

		if ( match ) {
			// disable the feature:

			if ( ch == '\0' )
				FeaturesList[base] = ch;
			else {
				j = 0;

				do {
					ch = FeaturesList[base + i + j];
					FeaturesList[base + j] = ch;
					j++;
				} while ( ch != '\0' );
			}
		}
		else
			base += i;

	}
	while ( ch != '\0' );

	i = wcslen(FeaturesList);
	if ( i > 0 )
		if ( ( FeaturesList[i - 1] == ',' ) || ( FeaturesList[i - 1] == ';' ) )
			FeaturesList[i - 1] = '\0';

	return 0;
}



BOOL APIENTRY DllMain( HANDLE hModule, DWORD  dwReason, LPVOID lpReserved )
{
	switch( dwReason )	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}



DLLExports LPPLUGININFO WINAPI GetPluginInfo( void )
{
	static PLUGININFO piInfo;

	piInfo.pszDll			= (LPTSTR) PLUGIN_NAME;
	piInfo.pszAuthor		= (LPTSTR) L"Charles Dye";
	piInfo.pszEmail			= (LPTSTR) L"cdye@unm.edu";
	piInfo.pszWWW			= (LPTSTR) L"http://charlesdye.net/plugins/osd.html";
	piInfo.pszDescription	= (LPTSTR) L"On-Screen Display";
	piInfo.pszFunctions		= (LPTSTR) PluginFeatures;
	piInfo.nMajor			= VER_MAJOR;
	piInfo.nMinor			= VER_MINOR;
	piInfo.nBuild			= VER_BUILD;

	return &piInfo;
}



DLLExports BOOL WINAPI InitializePlugin( void )
{
	if ( !OSD_Initialize() ) {
		if ( !PipeShell() ) {
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Error registering OSD window class!\r\n" );
			Qprintf( ERRHANDLE, L"%s plugin not loaded.\r\n", PLUGIN_NAME );
		}

		return -1;
	}

	wchar_t shortbuf[SHORT_BUF_LEN] = L"%_monitors";

	ExpandVariables( shortbuf, 1 );

	if ( iswdigit( shortbuf[0] ) )
		DisableOneFeature( PluginFeatures, L"_monitors" );


	wcscpy_s( shortbuf, SHORT_BUF_LEN, L"%[_transient]%[_pipe]%[noloadmsg]" );
	ExpandVariables( shortbuf, 1 );

	if ( wcscmp( shortbuf, L"00" ) == 0 )
		Printf( L"%s v%d.%d.%d loaded.\r\n", PLUGIN_NAME, VER_MAJOR, VER_MINOR, VER_BUILD );

	return 0;
}



DLLExports BOOL WINAPI ShutdownPlugin( BOOL bEndProcess )
{
	OSD_Shutdown();

	return 0;
}



DLLExports int WINAPI _monitors( LPTSTR String )
{
	if ( !String )
		return 1;


	wsprintf( String, L"%d", GetSystemMetrics( SM_CMONITORS ) );
	return 0;
}



DLLExports int WINAPI Help( LPTSTR inString )
{
	if ( _wcsicmp( inString, L"osd" ) == 0 ) {
		MessageBox( GetForegroundWindow(), OSDHelpText, HelpWindowTitle, MB_OK );
		return 1;
	}

#ifdef OSD_PCTBAR_CMD
	else if ( _wcsicmp( inString, L"pctbar" ) == 0 ) {
		MessageBox( GetForegroundWindow(), PctBarHelpText, HelpWindowTitle, MB_OK );
		return 1;
	}
#endif

#ifdef OSD_OSDIMG_CMD
	else if ( _wcsicmp( inString, L"osdimg" ) == 0 ) {
		MessageBox( GetForegroundWindow(), OsdImgHelpText, HelpWindowTitle, MB_OK );
		return 1;
	}
#endif

	else if ( _wcsicmp( inString, L"_osd" ) == 0 ) {
		MessageBox( GetForegroundWindow(), _OSDHelpText, HelpWindowTitle, MB_OK );
		return 1;
	}
	else if ( _wcsicmp( inString, L"_osdver" ) == 0 ) {
		MessageBox( GetForegroundWindow(), _OSDVerHelpText, HelpWindowTitle, MB_OK );
		return 1;
	}
	else if ( _wcsicmp( inString, L"@osd" ) == 0 ) {
		MessageBox( GetForegroundWindow(), F_OSDHelpText, HelpWindowTitle, MB_OK );
		return 1;
	}
	else
		return 0;
}



LPTSTR UsageText( const wchar_t *inString )
{
	if ( !inString )
		return NULL;


	wchar_t *ch = wcschr( (wchar_t*) inString, '\n' );


	if ( !ch )
		return (LPTSTR) inString;

	while ( *ch == '\n' )
		ch++;

	return ch;
}



DLLExports int WINAPI Usage( LPTSTR inString, LPTSTR outString )
{
	if ( !_wcsicmp( inString, L"osd" ) ) {
		wcscpy_s( outString, SHORT_BUF_LEN, UsageText( OSDHelpText ) );
		return 1;
	}

#ifdef OSD_PCTBAR_CMD

	else if ( !_wcsicmp( inString, L"pctbar" ) ) {
		wcscpy_s( outString, SHORT_BUF_LEN, UsageText( PctBarHelpText ) );
		return 1;
	}

#endif	//	 OSD_PCTBAR_CMD

#ifdef OSD_OSDIMG_CMD

	else if ( !_wcsicmp( inString, L"osdimg" ) ) {
		wcscpy_s( outString, SHORT_BUF_LEN, UsageText( OsdImgHelpText ) );
		return 1;
	}

#endif	//	OSD_OSDIMG_CMD

	else
		return 0;
}
